@@ -44,7 +44,7 @@ sub parse_factor
    my $self = shift;
 
    $self->any_of(
-      sub { $self->scope_of( "(", sub { $self->commit; $self->parse }, ")" ) },
+      sub { $self->committed_scope_of( "(", 'parse', ")" ) },
       sub { $self->token_int },
    );
 }
