@@ -1,14 +1,14 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2010-2015 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2010-2017 -- leonerd@leonerd.org.uk
 
 package Parser::MGC;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.15';
+our $VERSION = '0.16';
 
 use Carp;
 
@@ -317,6 +317,32 @@ sub pos
    return pos $self->{str};
 }
 
+=head2 take
+
+   $str = $parser->take( $len )
+
+I<Since version 0.16.>
+
+Returns the next C<$len> characters directly from the input, prior to any
+whitespace or comment skipping. This does I<not> take account of any pending
+end-of-scope marker that may be pending. It is intended for use by parsers of
+partially-binary protocols, or other situations in which it would be incorrect
+for the end-of-scope marker to take effect at this time.
+
+=cut
+
+sub take
+{
+   my $self = shift;
+   my ( $len ) = @_;
+
+   my $start = pos( $self->{str} );
+
+   pos( $self->{str} ) += $len;
+
+   return substr( $self->{str}, $start, $len );
+}
+
 =head2 where
 
    ( $lineno, $col, $text ) = $parser->where
@@ -540,12 +566,20 @@ immediately propagated to the caller.
 sub scope_of
 {
    my $self = shift;
-   my ( $start, $code, $stop ) = @_;
+   $self->_scope_of( 0, @_ );
+}
+
+sub _scope_of
+{
+   my $self = shift;
+   my ( $commit_if_started, $start, $code, $stop ) = @_;
 
    ref $stop or $stop = qr/\Q$stop/;
 
    $self->expect( $start ) if defined $start;
 
+   $self->commit if $commit_if_started;
+
    local $self->{endofscope} = $stop;
    local $self->{scope_level} = $self->{scope_level} + 1;
 
@@ -556,6 +590,25 @@ sub scope_of
    return $ret;
 }
 
+=head2 committed_scope_of
+
+   $ret = $parser->committed_scope_of( $start, $code, $stop )
+
+I<Since version 0.16.>
+
+A variant of L</scope_of> that calls L</commit> after a successful match of
+the start pattern. This is usually what you want if using C<scope_of> from
+within an C<any_of> choice, if no other alternative following this one could
+possibly match if the start pattern has.
+
+=cut
+
+sub committed_scope_of
+{
+   my $self = shift;
+   $self->_scope_of( 1, @_ );
+}
+
 =head2 list_of
 
    $ret = $parser->list_of( $sep, $code )
@@ -743,6 +796,8 @@ rather than by attempting other alternatives.
     );
  }
 
+Though in this common pattern, L</committed_scope_of> may be used instead.
+
 =cut
 
 sub commit
@@ -914,8 +969,7 @@ sub substring_before
       $end = length $self->{str};
    }
 
-   pos( $self->{str} ) = $end;
-   return substr( $self->{str}, $start, $end - $start );
+   return $self->take( $end - $start );
 }
 
 =head2 generic_token
